﻿using System;
using System.Collections.Generic;
using System.IO;
using Microsoft.SharePoint;
using Microsoft.SharePoint.Client;
using System.Collections;
using System.Text.RegularExpressions;

namespace MWG.SharePoint2010.DocumentService
{
    public class FileOperationService
    {
        ClientContext m_clientContext;

        void MoveVersions() 
        {
            SPFileCollection destFiles = null;
            SPFolder srcfolder = null;
            string sourceDocLib=string.Empty;
            //enumerate source library
            foreach (SPFile file in srcfolder.Files)
            {
                //standard check
                if (file.Exists)
                {
                    //custom sorted list used to reorder versions
                    SortedList myList = new SortedList();
                    ICollection items = myList.Keys;

                    //destination URL – path that will be used for copied file to target library including filename
                    string destURL = destFiles.Folder.Url + "/" + file.Name;

                    //checks to see if file has versions
                    if (file.Versions.Count != 0)
                    {
                        //enumerate versions
                        foreach (SPFileVersion ver in file.Versions)
                        {
                            string tempKey = "";
                            
                            //parses version number from previous versions URL
                            tempKey = Regex.Replace(ver.Url, "_vti_history/", "");
                            tempKey = Regex.Replace(tempKey, "/" + sourceDocLib, "");
                            tempKey = Regex.Replace(tempKey, "/" + file.Name, "");

                            //converts string number to int in order to be sorted correctly
                            //adds to Sorted list as a new key
                            myList.Add(int.Parse(tempKey), "");
                        }
                    }

                    //since items in sorted list are now actually sorted correctly
                    //we start with this list in order to process the versions
                    //to copy them in the correct order
                    foreach (object key in items)
                    {
                        //as we iterate the keys in the sorted list (the version numbers)
                        //we then run a comparison on the actual versions to find which one matches
                        //the key so we can process each one in order
                        foreach (SPFileVersion newVer in file.Versions)
                        {
                            string temp = "";

                            //parses version number from previous versions URL again 
                            //in order to compare it to key stored in SortedList.
                            temp = Regex.Replace(newVer.Url, "_vti_history/", "");
                            temp = Regex.Replace(temp, "/" + sourceDocLib, "");
                            temp = Regex.Replace(temp, "/" + file.Name, "");

                            //checks to see if version matches key 
                            if (temp == key.ToString())
                            {
                                //opens file for processing and calls method to determine major/minor status
                                byte[]  verFile = newVer.OpenBinary();
                                SetVersion(int.Parse(temp));
                            }
                        }
                    }

                    //Last step which copies current version
                    byte[] binFile;
                    SPFile copyFile;
                    binFile = file.OpenBinary();

                    //checks if current version is major/minor version and publishes accordingly
                    if (file.Level.ToString() == "Published")
                    {
                        copyFile = destFiles.Add(destURL, binFile, true);
                        copyFile.Publish("");
                    }
                    else
                    {
                        copyFile = destFiles.Add(destURL, binFile, true);
                    }
                }
            }

        }
        bool SetVersion(int num)
        {
            int baseNum = 512;
            decimal d = num / baseNum;
            int i = (int)Math.Floor(d) * 512;

            //major publish (eg 1.0, 2.0, 3.0)
            if (num == i)
            {
                return true;
                //copFileVers.Publish("");
            }
            //minor (eg 0.1, 1.1, 2.3)
            else
            {
                return false;
            }
        }

        void CopyWithoutVersion(SPFile spFile, string targetWeb, string targetUrl)
        {
            DiffSiteFileOperation op = new DiffSiteFileOperation(targetWeb);
            //Stream stream = spFile.OpenBinaryStream();
            ExFileInformation info = new ExFileInformation();
            info.ServerRelativePath = targetUrl;
            info.Name = spFile.Name;
            info.Content = spFile.OpenBinary();
            op.CopyFile(info);
        }

        void MoveWithVersion(string sourceDocLib, SPFile spFile, string targetWeb, string targetUrl,bool isEnableVersioning,bool keepDocumentID)
        {
            SortedList myList = new SortedList();
            ICollection items = myList.Keys; 

            DiffSiteFileOperation op = new DiffSiteFileOperation(targetWeb);
            ExFileInformation info = null;

            #region move with version
            if (isEnableVersioning && spFile.Versions.Count != 0)
            {
                //enumerate versions
                #region 
                foreach (SPFileVersion ver in spFile.Versions)
                {
                    string tempKey = "";

                    //parses version number from previous versions URL
                    tempKey = Regex.Replace(ver.Url, "_vti_history/", "");
                    tempKey = Regex.Replace(tempKey, "/" + sourceDocLib, "");
                    tempKey = Regex.Replace(tempKey, "/" + spFile.Name, "");
                    
                    //changed by johnny zhang
                    //using regex to match only numbers, because if there is to much history and the file in a folder, there is a int.parse error
                    Regex reg = new Regex(@"^\d+", RegexOptions.IgnoreCase);
                    MatchCollection matchs = reg.Matches(tempKey);
                    if (matchs.Count != 0)
                    {
                        //converts string number to int in order to be sorted correctly
                        //adds to Sorted list as a new key
                        myList.Add(int.Parse(matchs[0].ToString()), "");
                    }
                    else
                    {
                        continue;
                    }
                    //myList.Add(tempKey, "");
                }
                #endregion
                //since items in sorted list are now actually sorted correctly
                //we start with this list in order to process the versions
                //to copy them in the correct order
                foreach (object key in items)
                {
                    //as we iterate the keys in the sorted list (the version numbers)
                    //we then run a comparison on the actual versions to find which one matches
                    //the key so we can process each one in order
                    foreach (SPFileVersion newVer in spFile.Versions)
                    {
                        string temp = "";

                        //parses version number from previous versions URL again 
                        //in order to compare it to key stored in SortedList.
                        temp = Regex.Replace(newVer.Url, "_vti_history/", "");
                        temp = Regex.Replace(temp, "/" + sourceDocLib, "");
                        temp = Regex.Replace(temp, "/" + spFile.Name, "");
                        Regex reg = new Regex(@"^\d+", RegexOptions.IgnoreCase);
                        MatchCollection matchs = reg.Matches(temp);
                        if (matchs.Count != 0)
                        {
                            //converts string number to int in order to be sorted correctly
                            //adds to Sorted list as a new key
                            temp=matchs[0].ToString().Trim();
                        }
                      
                        //checks to see if version matches key 
                        if (temp == key.ToString())
                        {
                            info = new ExFileInformation();
                            //opens file for processing and calls method to determine major/minor status                           
                           // Stream stream = spFile.OpenBinaryStream();
                            info.ServerRelativePath = targetUrl;
                            info.Name = spFile.Name;
                            info.Content = newVer.OpenBinary();
                            info.IsMajorVersion = SetVersion(int.Parse(temp));
                            info.MWGDocumentID = spFile.Properties["_dlc_DocId"].ToString();
                            if (keepDocumentID)
                            {
                                info.KeepDocumnetID = true;
                            }
                            op.CopyFile(info);
                        }
                    }
                }
            }

            #endregion
            // copy current version.
            info = new ExFileInformation();
            //opens file for processing and calls method to determine major/minor status                           
            // Stream stream = spFile.OpenBinaryStream();
            info.ServerRelativePath = targetUrl;
            info.Name = spFile.Name;
            info.Content = spFile.OpenBinary();
            
            if (isEnableVersioning)
            {
                info.IsMajorVersion = true;
            }
            else
            {
                info.IsMajorVersion = false;
            }
            if (keepDocumentID)
            {
                info.MWGDocumentID = spFile.Properties["_dlc_DocId"].ToString();
                info.KeepDocumnetID = true;
            }
            op.CopyFile(info);
        }
        public void CopySingleFile(Guid listID, int itemID, String targetWeb, String targetUrl,bool isMove,bool keepDocumentID)
        {
            try
            {

                SPWeb web = SPContext.Current.Web;
                SPList currentList = web.Lists[listID];
                SPFile spFile = web.Lists[listID].GetItemById(itemID).File;

               
                if (spFile != null)
                {
                    #region old changed 2011-05-19
                    //    DiffSiteFileOperation op = new DiffSiteFileOperation(targetWeb);
                //    Stream stream = spFile.OpenBinaryStream();
                //    ExFileInformation info = new ExFileInformation();
                //    info.ServerRelativePath = targetUrl;
                //    info.Name = spFile.Name;
                //    info.Content = stream;
                    //    op.CopyFile(info);
                    #endregion
                    if (isMove)
                    {
                        bool isEnableVersioning = false;
                        isEnableVersioning = currentList.EnableVersioning;
                        MoveWithVersion(web.Lists[listID].Title, spFile, targetWeb, targetUrl, isEnableVersioning,keepDocumentID);
                    }
                    else
                    {
                        CopyWithoutVersion(spFile, targetWeb, targetUrl);
                    }
                }
               
            }
            catch (Exception ex)
            {
                throw new Exception("Copy single file error "+ex.Message);
            }

            return;
            //SPSecurity.RunWithElevatedPrivileges(delegate()
            //{
            //    try
            //    {
            //        using (SPSite site = new SPSite(SPContext.Current.Site.Url))
            //        {
            //            SPWeb web = site.OpenWeb();
            //            SPFile spFile = web.Lists[listID].GetItemById(itemID).File;
            //            if (spFile != null)
            //            {
            //                DiffSiteFileOperation op = new DiffSiteFileOperation(targetWeb);
            //                Stream stream = spFile.OpenBinaryStream();
            //                ExFileInformation info = new ExFileInformation();
            //                info.ServerRelativePath = targetUrl;
            //                info.Name = spFile.Name;
            //                info.Content = stream;
            //                op.CopyFile(info);
            //            }
            //        }
            //    }
            //    catch (Exception ex)
            //    {
            //        throw new Exception("Copy single file error");
            //    }

            //});

        }


        public String CreateFolder(String targetWeb, String targetUrl, String name)
        {


            DiffSiteFileOperation op = new DiffSiteFileOperation(targetWeb);
            ExFileInformation info = new ExFileInformation();
            //  SPSecurity.RunWithElevatedPrivileges(delegate()
            //{

            info.Name = name;
            info.ServerRelativePath = targetUrl;

            try
            {
                op.CopyFolder(info);
            }
            catch (Exception ex)
            {
                throw new Exception("Copy single Folder error "+ex.Message);
            }
            //});
            return info.ServerRelativePath + "/" + info.Name;
        }


        public String GetName(Guid listID, int itemID)
        {
            SPListItem item = null;
            //SPSecurity.RunWithElevatedPrivileges(delegate()

            try
            {
                // {
                item = SPContext.Current.Web.Lists[listID].GetItemById(itemID);
                //});
            }
            catch
            {
                throw new Exception("Get Name Error");
            }
            if (item.File == null)
            {
                return "Folder#;" + item.Folder.Name;
            }
            else
            {
                return "File#;" + item.File.Name;
            }
        }


        public List<int> GetFolderContents(Guid listID, int itemID)
        {
            List<int> result = null;
            //SPSecurity.RunWithElevatedPrivileges(delegate()
            //{

            result = new List<int>();

            try
            {
                SPFileCollection col = SPContext.Current.Web.Lists[listID].GetItemById(itemID).Folder.Files;

                foreach (SPFile itm in col)
                {
                    result.Add(itm.Item.ID);
                }
            }
            catch (Exception ex)
            {
                throw new Exception("Get folder contents Error! " +ex.Message);
            }
            //});

            return result;
        }


        public void DeleteSingleItem(Guid listID, int itemID)
        {

           
                using (SPSite site = SPContext.Current.Site)
                {
                    try
                    {
                        SPWeb web = site.OpenWeb();
                        web.AllowUnsafeUpdates = true;
                        SPListItem spListItem = web.Lists[listID].GetItemById(itemID);
                        SPFolder folder = spListItem.File.ParentFolder;
                        if (spListItem != null) spListItem.Delete();

                       
                        //if file's partent folder does not have any files, any subfolders
                        // and the file's parent folder is not the document library, then delete the parent folder

                        if (folder.Files.Count == 0 && folder.SubFolders.Count == 0 && folder.ParentFolder.DocumentLibrary != null)
                        {
                            folder.Delete();
                        }
                        SPContext.Current.Web.AllowUnsafeUpdates = false;
                    }
                    catch
                    {
                        throw new Exception("Delete item error");
                    }
                }
           
        }


        private void DeleteEmptyFolders(SPFolder sourceFolder)
        {
            List<int> deleteIDs = new List<int>();
            SPFolder folder = sourceFolder;


            //if the folder is empty, delete all folder
            if (hasFiles(folder) == false)
            {
                List<int> allIds = new List<int>();

                foreach (SPFolder subFolder in folder.SubFolders)
                {
                    if (subFolder.Name.ToUpper() != "FORMS")
                    {
                        allIds.Add(subFolder.Item.ID);
                    }
                }
                foreach (int itemID in allIds)
                {
                    SPListItem spListItem = SPContext.Current.Web.Lists[folder.ParentListId].GetItemById(itemID);

                    if (spListItem != null) spListItem.Delete();
                }
            }

            else // if there is some files in this folder
            {
                deleteIDs = DeleteEmptyFolder(folder);

                if (deleteIDs.Count != 0)
                {
                    foreach (int itemID in deleteIDs)
                    {
                        SPListItem spListItem = SPContext.Current.Web.Lists[folder.ParentListId].GetItemById(itemID);

                        if (spListItem != null) spListItem.Delete();
                    }
                }
            }

        }
        private bool hasFiles(SPFolder f)
        {

            if (f.Files.Count > 0)
                return true;
            else
            {
                if (f.SubFolders.Count > 0)
                {
                    foreach (SPFolder folder in f.SubFolders)
                    {
                        if (folder.Name.ToUpper() != "FORMS")
                        {
                            hasFiles(folder);
                        }
                    }
                }
            }

            return false;

        }

        private List<int> DeleteEmptyFolder(SPFolder folder)
        {
            List<int> ids = new List<int>();

            SPWeb web = SPContext.Current.Web;
            if (folder.Files.Count == 0)
            {
                foreach (SPFolder f in folder.SubFolders)
                {
                    if (f.Files.Count == 0 && f.SubFolders.Count == 0 && f.ParentFolder.DocumentLibrary != null)
                    {
                        ids.Add(f.Item.ID);
                    }
                    else
                    {
                        DeleteEmptyFolder(f);
                    }

                }
            }

            return ids;
        }

        public List<SPFolderObject> GetSubFolders(String url,string siteUrl)
        {
            List<SPFolderObject> list = null;

            try
            {
                //SPSecurity.RunWithElevatedPrivileges(delegate()
                //{
                //SPFolder folder = SPContext.Current.Web.GetFolder(url);
                //SPFolderCollection col = folder.SubFolders;
                //list = new List<SPFolderObject>();
                //foreach (SPFolder f in col)
                //{
                //    SPFolderObject o = new SPFolderObject();
                //    if (f.Item != null)
                //    {
                //        o.ID = f.Item.ID;
                //        o.ServerRelativeUrl = f.ServerRelativeUrl;
                //        o.Name = f.Name;
                //        list.Add(o);
                //    }
                //}
                using (SPSite site = new SPSite(siteUrl))
                {
                    using (SPWeb web = site.OpenWeb())
                    {
                        SPFolder folder = web.GetFolder(url);
                        SPFolderCollection col = folder.SubFolders;
                        list = new List<SPFolderObject>();
                        foreach (SPFolder f in col)
                        {
                            SPFolderObject o = new SPFolderObject();
                            if (f.Item != null)
                            {
                                o.ID = f.Item.ID;
                                o.ServerRelativeUrl = f.ServerRelativeUrl;
                                o.Name = f.Name;
                                list.Add(o);
                            }
                        }
                    }
                }
            }
            catch
            {
                throw new Exception("Get sub folders Error!");
            }
            //});
            return list;
        }
        /// <summary>
        /// get sub folder by parent name
        /// </summary>
        /// <param name="listID"></param>
        /// <param name="itemID"></param>
        /// <returns></returns>

        public List<SPFolderObject> GetSubFoldersByFolderName(Guid listID, int itemID)
        {
            List<SPFolderObject> list = new List<SPFolderObject>();
            //SPSecurity.RunWithElevatedPrivileges(delegate()
            //{

            try
            {
                SPListItem item = SPContext.Current.Web.Lists[listID].GetItemById(itemID);
                if (item != null && item.FileSystemObjectType == SPFileSystemObjectType.Folder)
                {
                    SPFolderCollection subFolders = item.Folder.SubFolders;

                    foreach (SPFolder f in subFolders)
                    {
                        SPFolderObject o = new SPFolderObject();
                        if (f.Item != null)
                        {
                            o.ID = f.Item.ID;
                            o.ServerRelativeUrl = f.ServerRelativeUrl;
                            o.Name = f.Name;
                            list.Add(o);
                        }
                    }
                }
            }
            catch
            {
                throw new Exception("Get sub folder(by folder name) error!");
            }
            //});
            return list;
        }
    }
    public class SPFolderObject
    {
        public String ServerRelativeUrl { get; set; }
        public int ID { get; set; }
        public String Name { get; set; }
    }
}