﻿using System;
using System.Threading;
using Microsoft.SharePoint.Client;

namespace MWG.SharePoint2010.DocumentService
{
    public class SameSiteFileOperator : FileOperatorBase
    {
        ClientContext _clientContext;
        AutoResetEvent autoResetEvent = new AutoResetEvent(false);

        public ClientContext ClientContext
        {
            get { return _clientContext; }
            set { _clientContext = value; }
        }

        public override bool CopyFile(string source, string destination)
        {
            if (!AutoOverwrite)
            {
                File destFile = _clientContext.Web.GetFileByServerRelativeUrl(destination);
                _clientContext.Load(destFile);
                try
                {
                    //if file does not exist, this will throw out an exception.
                    _clientContext.ExecuteQuery();

                    if (BeforeOverwrite.Method != null)
                    {
                        OverwriteEventArgs args = new OverwriteEventArgs();
                        args.FilePath = destination;
                        BeforeOverwrite.Invoke(this, args);
                        if (args.Stragety == OverwriteOptions.Cancel)
                            return false;
                        if (args.Stragety == OverwriteOptions.Skip)
                            return true;
                    }
                }
                catch { }
            }

            File srcFile = _clientContext.Web.GetFileByServerRelativeUrl(source);
            srcFile.CopyTo(destination, true);
            try
            {
                _clientContext.ExecuteQuery();
            }
            catch { }
            //}
            return true;
        }

        public override bool CopyFolder(string source, string destination)
        {
            if (destination.IndexOf(source) > -1) return true;
            Folder srcFolder = _clientContext.Web.GetFolderByServerRelativeUrl(source);
            Folder dstFolder = _clientContext.Web.GetFolderByServerRelativeUrl(destination);
            _clientContext.ExecuteQuery();
            String folderName = source.Substring(source.LastIndexOf('/') + 1);
            dstFolder.Folders.Add(folderName);
            try
            {
                _clientContext.ExecuteQuery();
            }
            catch { }
            FileCollection colFile = srcFolder.Files;
            _clientContext.Load(colFile);
            _clientContext.ExecuteQuery();
            foreach (File file in colFile)
            {
                bool isContinue = CopyFile(file.ServerRelativeUrl, destination + "/" + folderName + "/" + file.Name);
                if (!isContinue)
                {
                    return false;
                }
            }
            
            FolderCollection colFolder = srcFolder.Folders;
            _clientContext.Load(colFolder);
            _clientContext.ExecuteQuery();
            foreach (Folder folder in colFolder)
            {
                bool isContinue = CopyFolder(folder.ServerRelativeUrl, destination + "/" + folder.Name);
            }            
            return true;
        }

        public override bool MoveFile(string source, string destination)
        {
            if (!AutoOverwrite)
            {
                File destFile = _clientContext.Web.GetFileByServerRelativeUrl(destination);
                _clientContext.Load(destFile);
                try
                {
                    //if file does not exist, this will throw out an exception.
                    _clientContext.ExecuteQuery();

                    if (BeforeOverwrite.Method != null)
                    {
                        OverwriteEventArgs args = new OverwriteEventArgs();
                        args.FilePath = destination;
                        BeforeOverwrite.Invoke(this, args);
                        if (args.Stragety == OverwriteOptions.Cancel)
                            return false;
                        if (args.Stragety == OverwriteOptions.Skip)
                            return true;
                    }
                }
                catch { }
            }

            File srcFile = _clientContext.Web.GetFileByServerRelativeUrl(source);
            srcFile.MoveTo(destination, MoveOperations.Overwrite);
            try
            {
                _clientContext.ExecuteQuery();
            }
            catch { }
            //}
            return true;
        }

        public override bool MoveFolder(String source, String destination)
        {
            if (destination.IndexOf(source) > -1) return true;
            Folder srcFolder = _clientContext.Web.GetFolderByServerRelativeUrl(source);
            Folder dstFolder = _clientContext.Web.GetFolderByServerRelativeUrl(destination);
            _clientContext.ExecuteQuery();
            String folderName = source.Substring(source.LastIndexOf('/') + 1);
            dstFolder.Folders.Add(folderName);
            try
            {
                _clientContext.ExecuteQuery();
            }
            catch { }
            FileCollection colFile = srcFolder.Files;
            _clientContext.Load(colFile);
            _clientContext.ExecuteQuery();
            foreach (File file in colFile)
            {
                bool isContinue = CopyFile(file.ServerRelativeUrl, destination + "/" + folderName + "/" + file.Name);
                if (!isContinue)
                {
                    return false;
                }
            }

            FolderCollection colFolder = srcFolder.Folders;
            _clientContext.Load(colFolder);
            _clientContext.ExecuteQuery();
            foreach (Folder folder in colFolder)
            {
                bool isContinue = MoveFolder(folder.ServerRelativeUrl, destination + "/" + folder.Name);
            }

            colFolder.RefreshLoad();
            colFile.RefreshLoad();
            if (colFile.Count == 0 && colFolder.Count == 0)
            {
                srcFolder.DeleteObject();
                _clientContext.ExecuteQuery();
            }

            return true;
        }
        
    }
}
